package xyz.lychee.lagfixer.libs.adventure.platform.bukkit;

import com.google.common.collect.ImmutableList;
import com.google.common.graph.MutableGraph;
import java.lang.invoke.MethodHandle;
import java.lang.reflect.Field;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.Objects;
import java.util.function.BiConsumer;
import java.util.function.Consumer;
import java.util.function.Function;
import java.util.logging.Level;
import org.bukkit.Bukkit;
import org.bukkit.block.Block;
import org.bukkit.command.CommandSender;
import org.bukkit.command.ConsoleCommandSender;
import org.bukkit.command.ProxiedCommandSender;
import org.bukkit.entity.Entity;
import org.bukkit.entity.Player;
import org.bukkit.event.Event;
import org.bukkit.event.EventPriority;
import org.bukkit.event.Listener;
import org.bukkit.event.player.PlayerEvent;
import org.bukkit.event.player.PlayerJoinEvent;
import org.bukkit.event.player.PlayerQuitEvent;
import org.bukkit.plugin.Plugin;
import org.bukkit.plugin.PluginDescriptionFile;
import org.bukkit.plugin.PluginManager;
import xyz.lychee.lagfixer.libs.adventure.audience.Audience;
import xyz.lychee.lagfixer.libs.adventure.identity.Identity;
import xyz.lychee.lagfixer.libs.adventure.platform.facet.FacetAudienceProvider;
import xyz.lychee.lagfixer.libs.adventure.platform.facet.Knob;
import xyz.lychee.lagfixer.libs.adventure.pointer.Pointered;
import xyz.lychee.lagfixer.libs.adventure.text.flattener.ComponentFlattener;
import xyz.lychee.lagfixer.libs.adventure.text.renderer.ComponentRenderer;
import xyz.lychee.lagfixer.libs.adventure.translation.GlobalTranslator;
import xyz.lychee.lagfixer.libs.adventure.translation.Translator;
import xyz.lychee.lagfixer.libs.annotations.NotNull;
import xyz.lychee.lagfixer.libs.annotations.Nullable;

final class BukkitAudiencesImpl extends FacetAudienceProvider<CommandSender, BukkitAudience> implements BukkitAudiences, Listener {
   private static final Map<String, BukkitAudiences> INSTANCES;
   private final Plugin plugin;

   static Builder builder(final @NotNull Plugin plugin) {
      return new Builder(plugin);
   }

   static BukkitAudiences instanceFor(final @NotNull Plugin plugin) {
      return builder(plugin).build();
   }

   BukkitAudiencesImpl(final @NotNull Plugin plugin, final @NotNull ComponentRenderer<Pointered> componentRenderer) {
      super(componentRenderer);
      this.plugin = (Plugin)Objects.requireNonNull(plugin, "plugin");
      CommandSender console = this.plugin.getServer().getConsoleSender();
      this.addViewer(console);
      Iterator var4 = this.plugin.getServer().getOnlinePlayers().iterator();

      while(var4.hasNext()) {
         Player player = (Player)var4.next();
         this.addViewer(player);
      }

      this.registerEvent(PlayerJoinEvent.class, EventPriority.LOWEST, (event) -> {
         this.addViewer(event.getPlayer());
      });
      this.registerEvent(PlayerQuitEvent.class, EventPriority.MONITOR, (event) -> {
         this.removeViewer(event.getPlayer());
      });
      this.registerLocaleEvent(EventPriority.MONITOR, (viewer, locale) -> {
         BukkitAudience audience = (BukkitAudience)this.viewers.get(viewer);
         if (audience != null) {
            audience.locale(locale);
         }

      });
   }

   public @NotNull Audience sender(final @NotNull CommandSender sender) {
      if (sender instanceof Player) {
         return this.player((Player)sender);
      } else if (sender instanceof ConsoleCommandSender) {
         return this.console();
      } else if (sender instanceof ProxiedCommandSender) {
         return this.sender(((ProxiedCommandSender)sender).getCallee());
      } else {
         return (Audience)(!(sender instanceof Entity) && !(sender instanceof Block) ? this.createAudience(Collections.singletonList(sender)) : Audience.empty());
      }
   }

   public @NotNull Audience player(final @NotNull Player player) {
      return this.player(player.getUniqueId());
   }

   protected @NotNull BukkitAudience createAudience(final @NotNull Collection<CommandSender> viewers) {
      return new BukkitAudience(this.plugin, this, viewers);
   }

   public void close() {
      INSTANCES.remove(this.plugin.getName());
      super.close();
   }

   public @NotNull ComponentFlattener flattener() {
      return BukkitComponentSerializer.FLATTENER;
   }

   private <T extends Event> void registerEvent(final @NotNull Class<T> type, final @NotNull EventPriority priority, final @NotNull Consumer<T> callback) {
      Objects.requireNonNull(callback, "callback");
      this.plugin.getServer().getPluginManager().registerEvent(type, this, priority, (listener, event) -> {
         callback.accept(event);
      }, this.plugin, true);
   }

   private void registerLocaleEvent(final EventPriority priority, final @NotNull BiConsumer<Player, Locale> callback) {
      Class<?> eventClass = MinecraftReflection.findClass("org.bukkit.event.player.PlayerLocaleChangeEvent");
      if (eventClass == null) {
         eventClass = MinecraftReflection.findClass("com.destroystokyo.paper.event.player.PlayerLocaleChangeEvent");
      }

      MethodHandle getMethod = MinecraftReflection.findMethod(eventClass, "getLocale", String.class);
      if (getMethod == null) {
         getMethod = MinecraftReflection.findMethod(eventClass, "getNewLocale", String.class);
      }

      if (getMethod != null && PlayerEvent.class.isAssignableFrom(eventClass)) {
         Class<? extends PlayerEvent> localeEvent = eventClass;
         MethodHandle getLocale = getMethod;
         this.registerEvent(localeEvent, priority, (event) -> {
            Player player = event.getPlayer();

            String locale;
            try {
               locale = getLocale.invoke(event);
            } catch (Throwable var7) {
               Throwable error = var7;
               Knob.logError(error, "Failed to accept %s: %s", new Object[]{localeEvent.getName(), player});
               return;
            }

            callback.accept(player, toLocale(locale));
         });
      }

   }

   private static @NotNull Locale toLocale(final @Nullable String string) {
      if (string != null) {
         Locale locale = Translator.parseLocale(string);
         if (locale != null) {
            return locale;
         }
      }

      return Locale.US;
   }

   static {
      Knob.OUT = (message) -> {
         Bukkit.getLogger().log(Level.INFO, message);
      };
      Knob.ERR = (message, error) -> {
         Bukkit.getLogger().log(Level.WARNING, message, error);
      };
      INSTANCES = Collections.synchronizedMap(new HashMap(4));
   }

   static final class Builder implements BukkitAudiences.Builder {
      private final @NotNull Plugin plugin;
      private ComponentRenderer<Pointered> componentRenderer;

      Builder(final @NotNull Plugin plugin) {
         this.plugin = (Plugin)Objects.requireNonNull(plugin, "plugin");
         this.componentRenderer((ptr) -> {
            return (Locale)ptr.getOrDefault(Identity.LOCALE, BukkitAudiencesImpl.DEFAULT_LOCALE);
         }, GlobalTranslator.renderer());
      }

      @NotNull
      public @NotNull Builder componentRenderer(final @NotNull ComponentRenderer<Pointered> componentRenderer) {
         this.componentRenderer = (ComponentRenderer)Objects.requireNonNull(componentRenderer, "component renderer");
         return this;
      }

      public BukkitAudiences.@NotNull Builder partition(final @NotNull Function<Pointered, ?> partitionFunction) {
         Objects.requireNonNull(partitionFunction, "partitionFunction");
         return this;
      }

      public @NotNull BukkitAudiences build() {
         return (BukkitAudiences)BukkitAudiencesImpl.INSTANCES.computeIfAbsent(this.plugin.getName(), (name) -> {
            this.softDepend("ViaVersion");
            return new BukkitAudiencesImpl(this.plugin, this.componentRenderer);
         });
      }

      private void softDepend(final @NotNull String pluginName) {
         PluginDescriptionFile file = this.plugin.getDescription();
         if (!file.getName().equals(pluginName)) {
            try {
               Field softDepend = MinecraftReflection.needField(file.getClass(), "softDepend");
               List<String> dependencies = (List)softDepend.get(file);
               if (!dependencies.contains(pluginName)) {
                  List<String> newList = ImmutableList.builder().addAll(dependencies).add(pluginName).build();
                  softDepend.set(file, newList);
               }
            } catch (Throwable var7) {
               Throwable error = var7;
               Knob.logError(error, "Failed to inject softDepend in plugin.yml: %s %s", new Object[]{this.plugin, pluginName});
            }

            try {
               PluginManager manager = this.plugin.getServer().getPluginManager();
               Field dependencyGraphField = MinecraftReflection.needField(manager.getClass(), "dependencyGraph");
               MutableGraph<String> graph = (MutableGraph)dependencyGraphField.get(manager);
               graph.putEdge(file.getName(), pluginName);
            } catch (Throwable var6) {
            }

         }
      }
   }
}
